@@ -1,5 +1,6 @@
+from __future__ import annotations
+
 from dataclasses import dataclass, field
-from typing import List
 from enum import Enum
 
 
@@ -10,10 +11,10 @@ class TaskType(str, Enum):
     summarization = "summarization"
     machine_translation = "machine-translation"
     text_pair_classification = "text-pair-classification"
-    hellaswag = "hellaswag"
     aspect_based_sentiment_classification = "aspect-based-sentiment-classification"
     kg_link_tail_prediction = "kg-link-tail-prediction"
     qa_multiple_choice = "qa-multiple-choice"
+    conditional_generation = "conditional-generation"
 
     @staticmethod
     def list():
@@ -25,26 +26,29 @@ class Task:
     name: str
     description: str = "task description"
     supported: bool = field(default=False)
-    supported_metrics: List[str] = field(default_factory=list)
-    supported_formats: List[str] = field(default_factory=list)
-    supported_datasets: List[str] = field(default_factory=list)
+    supported_metrics: list[str] = field(default_factory=list)
+    supported_formats: list[str] = field(default_factory=list)
+    supported_datasets: list[str] = field(default_factory=list)
 
 
 @dataclass
 class TaskCategory:
     name: str
     description: str
-    tasks: List[Task]
+    tasks: list[Task]
 
 
-_task_categories: List[TaskCategory] = [
+_task_categories: list[TaskCategory] = [
     TaskCategory(
         "conditional-text-generation",
         "data-to-text and text transduction tasks such as translation or summarization",
         [
             Task(
                 name=TaskType.machine_translation,
-                description="The process of using AI to automatically translate text from one language to another.",
+                description=(
+                    "The process of using AI to automatically translate text from one "
+                    "language to another."
+                ),
                 supported=True,
                 supported_metrics=[
                     "bleu",
@@ -67,7 +71,37 @@ _task_categories: List[TaskCategory] = [
             ),
             Task(
                 name=TaskType.summarization,
-                description="Summarize long documents into short texts. See more details about the format of upload files: https://github.com/neulab/ExplainaBoard/blob/main/docs/task_summarization.md",
+                description="""
+Summarize long documents into short texts.
+See more details about the format of upload files:
+https://github.com/neulab/ExplainaBoard/blob/main/docs/task_summarization.md
+""",
+                supported=True,
+                supported_metrics=[
+                    "bleu",
+                    "bart_score_summ",
+                    "bart_score_mt",
+                    "bart_score_cnn_hypo_ref",
+                    "rouge1",
+                    "rouge2",
+                    "rougeL",
+                    "bert_score_f",
+                    "bert_score_p",
+                    "bert_score_r",
+                    "chrf",
+                    "comet",
+                    "mover_score",
+                    "prism",
+                ],
+                supported_formats=["tsv"],
+                supported_datasets=[],
+            ),
+            Task(
+                name=TaskType.conditional_generation,
+                description=(
+                    "Generic conditional text generation tasks, e.g., machine "
+                    "translation, text summarization"
+                ),
                 supported=True,
                 supported_metrics=[
                     "bleu",
@@ -96,9 +130,13 @@ _task_categories: List[TaskCategory] = [
         [
             Task(
                 name=TaskType.text_classification,
-                description="Classify a text into one or multiple predefined categories. See more details about the format of upload files: https://github.com/neulab/ExplainaBoard/blob/main/docs/task_text_classification.md",
+                description="""
+Classify a text into one or multiple predefined categories.
+See more details about the format of upload files:
+https://github.com/neulab/ExplainaBoard/blob/main/docs/task_text_classification.md
+""",
                 supported=True,
-                supported_metrics=["F1score", "Accuracy"],
+                supported_metrics=["F1Score", "Accuracy"],
                 supported_formats=["tsv"],
                 supported_datasets=[],
             )
@@ -110,9 +148,13 @@ _task_categories: List[TaskCategory] = [
         [
             Task(
                 name=TaskType.named_entity_recognition,
-                description="Recognize named entities from a given text. See one example of the uploaded file: https://github.com/neulab/ExplainaBoard/blob/main/data/system_outputs/conll2003/conll2003.elmo",
+                description="""
+Recognize named entities from a given text.
+See one example of the uploaded file:
+https://github.com/neulab/ExplainaBoard/blob/main/data/system_outputs/conll2003/conll2003.elmo
+""",
                 supported=True,
-                supported_metrics=["f1_score_seqeval"],
+                supported_metrics=["f1_seqeval", "recall_seqeval", "precision_seqeval"],
                 supported_formats=["conll"],
                 supported_datasets=[],
             )
@@ -124,9 +166,13 @@ _task_categories: List[TaskCategory] = [
         [
             Task(
                 name=TaskType.question_answering_extractive,
-                description="A task of extracting an answer from a text given a question. See more details about the format of upload files: https://github.com/neulab/ExplainaBoard/blob/main/docs/task_extractive_qa_squad.md",
+                description="""
+A task of extracting an answer from a text given a question.
+See more details about the format of upload files:
+https://github.com/neulab/ExplainaBoard/blob/main/docs/task_extractive_qa_squad.md
+""",
                 supported=True,
-                supported_metrics=["f1_score_qa", "exact_match_qa"],
+                supported_metrics=["F1ScoreQA", "ExactMatchQA"],
                 supported_formats=["json"],
                 supported_datasets=[],
             ),
@@ -138,9 +184,13 @@ _task_categories: List[TaskCategory] = [
         [
             Task(
                 name=TaskType.aspect_based_sentiment_classification,
-                description="Predict the sentiment of a text based on a specific aspect. See more details about the format of upload files: https://github.com/neulab/ExplainaBoard/blob/main/data/system_outputs/absa/test-aspect.tsv",
+                description="""
+Predict the sentiment of a text based on a specific aspect.
+See more details about the format of upload files:
+https://github.com/neulab/ExplainaBoard/blob/main/data/system_outputs/absa/test-aspect.tsv
+""",
                 supported=True,
-                supported_metrics=["F1score", "Accuracy"],
+                supported_metrics=["F1Score", "Accuracy"],
                 supported_formats=["tsv"],
                 supported_datasets=[],
             ),
@@ -152,9 +202,13 @@ _task_categories: List[TaskCategory] = [
         [
             Task(
                 name=TaskType.text_pair_classification,
-                description="predict the relationship of two texts. See more details about the format of upload files: https://github.com/neulab/ExplainaBoard/blob/main/docs/task_text_pair_classification.md",
+                description="""
+predict the relationship of two texts.
+See more details about the format of upload files:
+https://github.com/neulab/ExplainaBoard/blob/main/docs/task_text_pair_classification.md
+""",
                 supported=True,
-                supported_metrics=["F1score", "Accuracy"],
+                supported_metrics=["F1Score", "Accuracy"],
                 supported_formats=["tsv"],
                 supported_datasets=[],
             ),
@@ -166,7 +220,11 @@ _task_categories: List[TaskCategory] = [
         [
             Task(
                 name=TaskType.kg_link_tail_prediction,
-                description="predicting the tail entity of missing links in knowledge graphs. See more details about the format of upload files: https://github.com/neulab/ExplainaBoard/blob/main/docs/task_kg_link_tail_prediction.md",
+                description="""
+predicting the tail entity of missing links in knowledge graphs.
+See more details about the format of upload files:
+https://github.com/neulab/ExplainaBoard/blob/main/docs/task_kg_link_tail_prediction.md
+""",
                 supported=True,
                 supported_metrics=["Hits", "MeanReciprocalRank"],
                 supported_formats=["json"],
@@ -182,7 +240,7 @@ _task_categories: List[TaskCategory] = [
                 name=TaskType.qa_multiple_choice,
                 description="Answer a question from multiple options",
                 supported=True,
-                supported_metrics=["F1score", "Accuracy"],
+                supported_metrics=["F1Score", "Accuracy"],
                 supported_formats=["json"],
                 supported_datasets=[],
             )
