@@ -1,8 +1,8 @@
-import pathlib
 import os
+import pathlib
 import unittest
-import tqdm
-from explainaboard import FileType, Source, TaskType, get_loader, get_processor
+
+from explainaboard import FileType, get_loader, get_processor, Source, TaskType
 from explainaboard.tests.utils import load_file_as_str
 
 artifacts_path = os.path.dirname(pathlib.Path(__file__)) + "/artifacts/"
@@ -12,23 +12,19 @@ class TestAspectBasedSentimentClassification(unittest.TestCase):
     def test_e2e(self):
         metadata = {
             "task_name": TaskType.aspect_based_sentiment_classification.value,
-            "metric_names": ["Accuracy", "F1score"],
+            "metric_names": ["Accuracy", "F1Score"],
         }
         loader = get_loader(
             TaskType.aspect_based_sentiment_classification,
+            load_file_as_str(f"{artifacts_path}test-aspect.tsv"),
             Source.in_memory,
             FileType.tsv,
-            load_file_as_str(f"{artifacts_path}test-aspect.tsv"),
         )
         data = loader.load()
-        processor = get_processor(
-            TaskType.aspect_based_sentiment_classification, metadata, data
-        )
-        self.assertEqual(len(processor._features), 10)
+        processor = get_processor(TaskType.aspect_based_sentiment_classification)
 
-        analysis = processor.process()
+        sys_info = processor.process(metadata, data)
 
         # analysis.write_to_directory("./")
-        self.assertListEqual(analysis.metric_names, metadata["metric_names"])
-        self.assertIsNotNone(analysis.results.fine_grained)
-        self.assertGreater(len(analysis.results.overall), 0)
+        self.assertIsNotNone(sys_info.results.fine_grained)
+        self.assertGreater(len(sys_info.results.overall), 0)
