@@ -1,7 +1,8 @@
-import pathlib
 import os
+import pathlib
 import unittest
-from explainaboard import FileType, Source, TaskType, get_loader, get_processor
+
+from explainaboard import FileType, get_loader, get_processor, Source, TaskType
 
 artifacts_path = os.path.dirname(pathlib.Path(__file__)) + "/artifacts/"
 
@@ -13,9 +14,9 @@ class TestNER(unittest.TestCase):
         path_data = artifacts_path + "test-ner.tsv"
         loader = get_loader(
             TaskType.named_entity_recognition,
+            path_data,
             Source.local_filesystem,
             FileType.conll,
-            path_data,
         )
         data = loader.load()
 
@@ -23,18 +24,16 @@ class TestNER(unittest.TestCase):
             "task_name": TaskType.named_entity_recognition.value,
             # "dataset_name": "conll2003",
             # "sub_dataset_name":"ner",
-            "metric_names": ["f1_score_seqeval"],
+            "metric_names": ["F1Score"],
         }
 
-        processor = get_processor(TaskType.named_entity_recognition, metadata, data)
-        # self.assertEqual(len(processor._features), 4)
+        processor = get_processor(TaskType.named_entity_recognition)
 
-        analysis = processor.process()
-        analysis.write_to_directory("./")
+        sys_info = processor.process(metadata, data)
 
-        self.assertListEqual(analysis.metric_names, metadata["metric_names"])
-        self.assertIsNotNone(analysis.results.fine_grained)
-        self.assertGreater(len(analysis.results.overall), 0)
+        # analysis.write_to_directory("./")
+        self.assertIsNotNone(sys_info.results.fine_grained)
+        self.assertGreater(len(sys_info.results.overall), 0)
 
 
 if __name__ == '__main__':
